function [xfilt,varargout] = kf_smooth(y, A, B, C, d, u, Q, R, init_x, init_V)
%
% function [xfilt, Vfilt, loglik, xsmooth, Vsmooth, Q, R] = 
%           kf_smooth(y, A, B, C, d, u, Q, R, init_x, init_V)
%
%
% Kalman filter
% [xfilt, Vfilt] = ekf_smooth(y_all, A, B, C, d, Q, R, init_x, init_V);
%
% Kalman filter with Smoother
% [xfilt, Vfilt, loglik, xsmooth, Vsmooth] = ekf_smooth(y_all, A, B, C, d, Q, R, init_x, init_V);
%
% Kalman filter with Smoother and EM algorithm
% [xfilt, Vfilt, loglik, xsmooth, Vsmooth, Q, R] = ekf_smooth(y_all, A, B, C, d, Q, R, init_x, init_V);
%
%
% INPUTS:
% y - observations
% A, B, C, d:  x(:,t+1) = A x(:,t) + B u(:,t) + w(:,t) 
%              y(:,t)   = C x(:,t) + d        + v(:,t)
% Q - covariance matrix of system x(t+1)=A*x(t)+w(t) , w(t)~N(0,Q)
% R - covariance matrix of output y(t)=C*x(t)+v(t) , v(t)~N(0,R)
% init_x -
% init_V -
%
%
% OUTPUTS:
% xfilt = E[X_t|t]
% varargout(1) = Vfilt - Cov[X_t|0:t]
% varargout(2) = loglik - loglikelihood
% varargout(3) = xsmooth - E[X_t|0:T]
% varargout(4) = Vsmooth - Cov[X_t|0:T]
% varargout(5) = Q - estimated system covariance according to 1 EM update
% varargout(6) = R - estimated output covariance according to 1 EM update

n_var_out = max(nargout,1)-1; % number of variable number of outputs

T = size(y,2);
ss = size(Q,1); % size of state space

%% Forward pass (Filter)

%YOUR code here

if(n_var_out >= 1), varargout(1) = {Vfilt}; end
if(n_var_out >= 2), varargout(2) = {loglik}; end


%% Backward pass (RTS Smoother and EM algorithm)
if(n_var_out >= 3)

	%YOUR code here
	
	varargout(3) = {xsmooth};
   if(n_var_out >= 4), varargout(4) = {Vsmooth}; end
   if(n_var_out >= 5), varargout(5) = {Q}; end
   if(n_var_out == 6), varargout(6) = {R}; end
end





















